@@ -12,8 +12,14 @@
 #ifndef XSIMD_INLINE_HPP
 #define XSIMD_INLINE_HPP
 
-#if defined(__GNUC__)
+#if defined(__VEC__)
+#define XSIMD_INLINE inline
+#elif defined __has_attribute
+#if __has_attribute(always_inline)
 #define XSIMD_INLINE inline __attribute__((always_inline))
+#else
+#define XSIMD_INLINE inline
+#endif
 #elif defined(_MSC_VER)
 #define XSIMD_INLINE inline __forceinline
 #else
