@@ -101,6 +101,7 @@ struct batch_test
     {
         batch_type b;
         // value initialized to random data, can't be checked
+        (void)b;
 
         array_type tmp;
         std::fill(tmp.begin(), tmp.end(), value_type(2));
@@ -151,6 +152,12 @@ struct batch_test
         }
     }
 
+    void test_first_element() const
+    {
+        batch_type res = batch_lhs();
+        CHECK_EQ(res.first(), lhs[0]);
+    }
+
     void test_arithmetic() const
     {
         // +batch
@@ -248,6 +255,49 @@ struct batch_test
         }
     }
 
+    void test_incr_decr() const
+    {
+        // incr
+        {
+            array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), expected.begin(), xsimd::incr<value_type>);
+            batch_type res = xsimd::incr(batch_lhs());
+            INFO("incr(batch)");
+            CHECK_BATCH_EQ(res, expected);
+        }
+
+        // incr_if
+        {
+            array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), expected.begin(),
+                           [](value_type v)
+                           { return v > 1 ? v + 1 : v; });
+            batch_type res = xsimd::incr_if(batch_lhs(), batch_lhs() > value_type(1));
+            INFO("incr_if(batch)");
+            CHECK_BATCH_EQ(res, expected);
+        }
+
+        // decr
+        {
+            array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), expected.begin(), xsimd::decr<value_type>);
+            batch_type res = xsimd::decr(batch_lhs());
+            INFO("decr(batch)");
+            CHECK_BATCH_EQ(res, expected);
+        }
+
+        // decr_if
+        {
+            array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), expected.begin(),
+                           [](value_type v)
+                           { return v > 1 ? v - 1 : v; });
+            batch_type res = xsimd::decr_if(batch_lhs(), batch_lhs() > value_type(1));
+            INFO("decr_if(batch)");
+            CHECK_BATCH_EQ(res, expected);
+        }
+    }
+
     void test_saturated_arithmetic() const
     {
         // batch + batch
@@ -427,6 +477,11 @@ struct batch_test
             auto res = batch_lhs() < batch_rhs();
             INFO("batch < batch");
             CHECK_BATCH_EQ(res, expected);
+
+            std::fill(expected.begin(), expected.end(), false);
+            res = batch_lhs() < batch_lhs();
+            INFO("batch < (self)");
+            CHECK_BATCH_EQ(res, expected);
         }
         // batch < scalar
         {
@@ -452,6 +507,11 @@ struct batch_test
             auto res = batch_lhs() <= batch_rhs();
             INFO("batch <= batch");
             CHECK_BATCH_EQ(res, expected);
+
+            std::fill(expected.begin(), expected.end(), true);
+            res = batch_lhs() <= batch_lhs();
+            INFO("batch < (self)");
+            CHECK_BATCH_EQ(res, expected);
         }
         // batch <= scalar
         {
@@ -477,6 +537,11 @@ struct batch_test
             auto res = batch_lhs() > batch_rhs();
             INFO("batch > batch");
             CHECK_BATCH_EQ(res, expected);
+
+            std::fill(expected.begin(), expected.end(), false);
+            res = batch_lhs() > batch_lhs();
+            INFO("batch > (self)");
+            CHECK_BATCH_EQ(res, expected);
         }
         // batch > scalar
         {
@@ -501,6 +566,11 @@ struct batch_test
             auto res = batch_lhs() >= batch_rhs();
             INFO("batch >= batch");
             CHECK_BATCH_EQ(res, expected);
+
+            std::fill(expected.begin(), expected.end(), true);
+            res = batch_lhs() >= batch_lhs();
+            INFO("batch >= (self)");
+            CHECK_BATCH_EQ(res, expected);
         }
         // batch >= scalar
         {
@@ -667,6 +737,20 @@ struct batch_test
             INFO("fnms");
             CHECK_BATCH_EQ(res, expected);
         }
+        // fmas
+        {
+            array_type expected;
+            for (std::size_t i = 0; i < expected.size(); ++i)
+            {
+                // even lanes: x*y - z, odd lanes: x*y + z
+                expected[i] = (i & 1u) == 0
+                    ? lhs[i] * rhs[i] - rhs[i]
+                    : lhs[i] * rhs[i] + rhs[i];
+            }
+            batch_type res = fmas(batch_lhs(), batch_rhs(), batch_rhs());
+            INFO("fmas");
+            CHECK_BATCH_EQ(res, expected);
+        }
     }
 
     void test_abs() const
@@ -693,6 +777,46 @@ struct batch_test
         }
     }
 
+    void test_avg() const
+    {
+        {
+            array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
+                           [](const value_type& l, const value_type& r) -> value_type
+                           {
+                               if (std::is_integral<value_type>::value)
+                               {
+                                   return static_cast<value_type>(((long long)l + r) / 2);
+                               }
+                               else
+                               {
+                                   return (l + r) / 2;
+                               }
+                           });
+            batch_type res = avg(batch_lhs(), batch_rhs());
+            INFO("avg");
+            CHECK_BATCH_EQ(res, expected);
+        }
+        {
+            array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
+                           [](const value_type& l, const value_type& r) -> value_type
+                           {
+                               if (std::is_integral<value_type>::value)
+                               {
+                                   return static_cast<value_type>(((long long)l + r) / 2 + ((long long)(l + r) & 1));
+                               }
+                               else
+                               {
+                                   return (l + r) / 2;
+                               }
+                           });
+            batch_type res = avgr(batch_lhs(), batch_rhs());
+            INFO("avgr");
+            CHECK_BATCH_EQ(res, expected);
+        }
+    }
+
     void test_horizontal_operations() const
     {
         // reduce_add
@@ -716,8 +840,28 @@ struct batch_test
             INFO("reduce_min");
             CHECK_SCALAR_EQ(res, expected);
         }
+        // reduce_mul
+        {
+            value_type expected = std::accumulate(lhs.cbegin(), lhs.cend(), value_type(1), std::multiplies<value_type>());
+            value_type res = reduce_mul(batch_lhs());
+            INFO("reduce_mul");
+            CHECK_SCALAR_EQ(res, expected);
+        }
     }
 
+    template <size_t N>
+    typename std::enable_if<4 <= N>::type test_common_horizontal_operations(std::integral_constant<size_t, N>) const
+    {
+        // reduce common
+        {
+            value_type expected = std::accumulate(lhs.cbegin(), lhs.cend(), value_type(1), std::multiplies<value_type>());
+            value_type res = reduce(xsimd::mul<typename B::value_type, typename B::arch_type>, batch_lhs());
+            INFO("common reduce");
+            CHECK_SCALAR_EQ(res, expected);
+        }
+    }
+    void test_common_horizontal_operations(...) const { }
+
     void test_boolean_conversions() const
     {
         using batch_bool_type = typename batch_type::batch_bool_type;
@@ -782,12 +926,12 @@ private:
         for (size_t i = 0; i < size; ++i)
         {
             bool negative_lhs = std::is_signed<T>::value && (i % 2 == 1);
-            lhs[i] = value_type(i) * (negative_lhs ? -10 : 10);
+            lhs[i] = value_type(i) * (negative_lhs ? -3 : 3);
             if (lhs[i] == value_type(0))
             {
                 lhs[i] += value_type(1);
             }
-            rhs[i] = value_type(i) + value_type(4);
+            rhs[i] = value_type(i) + value_type(2);
         }
         scalar = value_type(3);
     }
@@ -837,11 +981,21 @@ TEST_CASE_TEMPLATE("[batch]", B, BATCH_TYPES)
         Test.test_access_operator();
     }
 
+    SUBCASE("first element")
+    {
+        Test.test_first_element();
+    }
+
     SUBCASE("arithmetic")
     {
         Test.test_arithmetic();
     }
 
+    SUBCASE("incr decr")
+    {
+        Test.test_incr_decr();
+    }
+
     SUBCASE("saturated_arithmetic")
     {
         Test.test_saturated_arithmetic();
@@ -876,9 +1030,15 @@ TEST_CASE_TEMPLATE("[batch]", B, BATCH_TYPES)
         Test.test_abs();
     }
 
+    SUBCASE("avg")
+    {
+        Test.test_avg();
+    }
+
     SUBCASE("horizontal_operations")
     {
         Test.test_horizontal_operations();
+        Test.test_common_horizontal_operations(std::integral_constant<size_t, sizeof(typename B::value_type)>());
     }
 
     SUBCASE("boolean_conversions")
