@@ -19,6 +19,7 @@ struct constant_batch_test
 {
     using batch_type = B;
     using value_type = typename B::value_type;
+    using arch_type = typename B::arch_type;
     static constexpr size_t size = B::size;
     using array_type = std::array<value_type, size>;
     using bool_array_type = std::array<bool, size>;
@@ -32,6 +33,16 @@ struct constant_batch_test
         }
     };
 
+    void test_init_from_constant() const
+    {
+        array_type expected;
+        std::generate(expected.begin(), expected.end(), []()
+                      { return 1; });
+        constexpr auto b = xsimd::make_batch_constant<value_type, 1, arch_type>();
+        INFO("batch(value_type)");
+        CHECK_BATCH_EQ((batch_type)b, expected);
+    }
+
     void test_init_from_generator() const
     {
         array_type expected;
@@ -39,16 +50,25 @@ struct constant_batch_test
         std::generate(expected.begin(), expected.end(),
                       [&i]()
                       { return generator::get(i++, size); });
-        constexpr auto b = xsimd::make_batch_constant<batch_type, generator>();
+        constexpr auto b = xsimd::make_batch_constant<value_type, generator, arch_type>();
         INFO("batch(value_type)");
         CHECK_BATCH_EQ((batch_type)b, expected);
     }
 
+    void test_cast() const
+    {
+        constexpr auto cst_b = xsimd::make_batch_constant<value_type, generator, arch_type>();
+        auto b0 = cst_b.as_batch();
+        auto b1 = (batch_type)cst_b;
+        CHECK_BATCH_EQ(b0, b1);
+        // The actual values are already tested in test_init_from_generator
+    }
+
     struct arange
     {
         static constexpr value_type get(size_t index, size_t /*size*/)
         {
-            return index;
+            return static_cast<value_type>(index);
         }
     };
 
@@ -59,7 +79,7 @@ struct constant_batch_test
         std::generate(expected.begin(), expected.end(),
                       [&i]()
                       { return arange::get(i++, size); });
-        constexpr auto b = xsimd::make_batch_constant<batch_type, arange>();
+        constexpr auto b = xsimd::make_batch_constant<value_type, arange, arch_type>();
         INFO("batch(value_type)");
         CHECK_BATCH_EQ((batch_type)b, expected);
     }
@@ -73,38 +93,38 @@ struct constant_batch_test
         }
     };
 
-    void test_init_from_constant() const
+    void test_init_from_constant_generator() const
     {
         array_type expected;
         std::fill(expected.begin(), expected.end(), constant<3>::get(0, 0));
-        constexpr auto b = xsimd::make_batch_constant<batch_type, constant<3>>();
+        constexpr auto b = xsimd::make_batch_constant<value_type, constant<3>, arch_type>();
         INFO("batch(value_type)");
         CHECK_BATCH_EQ((batch_type)b, expected);
     }
 
     void test_ops() const
     {
-        constexpr auto n12 = xsimd::make_batch_constant<batch_type, constant<12>>();
-        constexpr auto n3 = xsimd::make_batch_constant<batch_type, constant<3>>();
+        constexpr auto n12 = xsimd::make_batch_constant<value_type, constant<12>, arch_type>();
+        constexpr auto n3 = xsimd::make_batch_constant<value_type, constant<3>, arch_type>();
 
         constexpr auto n12_add_n3 = n12 + n3;
-        constexpr auto n15 = xsimd::make_batch_constant<batch_type, constant<15>>();
+        constexpr auto n15 = xsimd::make_batch_constant<value_type, constant<15>, arch_type>();
         static_assert(std::is_same<decltype(n12_add_n3), decltype(n15)>::value, "n12 + n3 == n15");
 
         constexpr auto n12_sub_n3 = n12 - n3;
-        constexpr auto n9 = xsimd::make_batch_constant<batch_type, constant<9>>();
+        constexpr auto n9 = xsimd::make_batch_constant<value_type, constant<9>, arch_type>();
         static_assert(std::is_same<decltype(n12_sub_n3), decltype(n9)>::value, "n12 - n3 == n9");
 
         constexpr auto n12_mul_n3 = n12 * n3;
-        constexpr auto n36 = xsimd::make_batch_constant<batch_type, constant<36>>();
+        constexpr auto n36 = xsimd::make_batch_constant<value_type, constant<36>, arch_type>();
         static_assert(std::is_same<decltype(n12_mul_n3), decltype(n36)>::value, "n12 * n3 == n36");
 
         constexpr auto n12_div_n3 = n12 / n3;
-        constexpr auto n4 = xsimd::make_batch_constant<batch_type, constant<4>>();
+        constexpr auto n4 = xsimd::make_batch_constant<value_type, constant<4>, arch_type>();
         static_assert(std::is_same<decltype(n12_div_n3), decltype(n4)>::value, "n12 / n3 == n4");
 
         constexpr auto n12_mod_n3 = n12 % n3;
-        constexpr auto n0 = xsimd::make_batch_constant<batch_type, constant<0>>();
+        constexpr auto n0 = xsimd::make_batch_constant<value_type, constant<0>, arch_type>();
         static_assert(std::is_same<decltype(n12_mod_n3), decltype(n0)>::value, "n12 % n3 == n0");
 
         constexpr auto n12_land_n3 = n12 & n3;
@@ -116,30 +136,66 @@ struct constant_batch_test
         constexpr auto n12_lxor_n3 = n12 ^ n3;
         static_assert(std::is_same<decltype(n12_lxor_n3), decltype(n15)>::value, "n12 ^ n3 == n15");
 
+        constexpr auto n96 = xsimd::make_batch_constant<value_type, constant<96>, arch_type>();
+        constexpr auto n12_lshift_n3 = n12 << n3;
+        static_assert(std::is_same<decltype(n12_lshift_n3), decltype(n96)>::value, "n12 << n3 == n96");
+
+        constexpr auto n1 = xsimd::make_batch_constant<value_type, constant<1>, arch_type>();
+        constexpr auto n12_rshift_n3 = n12 >> n3;
+        static_assert(std::is_same<decltype(n12_rshift_n3), decltype(n1)>::value, "n12 >> n3 == n1");
+
         constexpr auto n12_uadd = +n12;
         static_assert(std::is_same<decltype(n12_uadd), decltype(n12)>::value, "+n12 == n12");
 
         constexpr auto n12_inv = ~n12;
-        constexpr auto n12_inv_ = xsimd::make_batch_constant<batch_type, constant<(value_type)~12>>();
+        constexpr auto n12_inv_ = xsimd::make_batch_constant<value_type, constant<(value_type)~12>, arch_type>();
         static_assert(std::is_same<decltype(n12_inv), decltype(n12_inv_)>::value, "~n12 == n12_inv");
 
         constexpr auto n12_usub = -n12;
-        constexpr auto n12_usub_ = xsimd::make_batch_constant<batch_type, constant<(value_type)-12>>();
+        constexpr auto n12_usub_ = xsimd::make_batch_constant<value_type, constant<(value_type)-12>, arch_type>();
         static_assert(std::is_same<decltype(n12_usub), decltype(n12_usub_)>::value, "-n12 == n12_usub");
+
+        // comparison operators
+        using true_batch_type = decltype(xsimd::make_batch_bool_constant<value_type, true, arch_type>());
+        using false_batch_type = decltype(xsimd::make_batch_bool_constant<value_type, false, arch_type>());
+
+        static_assert(std::is_same<typename decltype(n12 == n12)::operand_type, typename decltype(n12)::value_type>::value, "same type");
+
+        static_assert(std::is_same<decltype(n12 == n12), true_batch_type>::value, "n12 == n12");
+        static_assert(std::is_same<decltype(n12 == n3), false_batch_type>::value, "n12 == n3");
+
+        static_assert(std::is_same<decltype(n12 != n12), false_batch_type>::value, "n12 != n12");
+        static_assert(std::is_same<decltype(n12 != n3), true_batch_type>::value, "n12 != n3");
+
+        static_assert(std::is_same<decltype(n12 < n12), false_batch_type>::value, "n12 < n12");
+        static_assert(std::is_same<decltype(n12 < n3), false_batch_type>::value, "n12 < n3");
+
+        static_assert(std::is_same<decltype(n12 > n12), false_batch_type>::value, "n12 > n12");
+        static_assert(std::is_same<decltype(n12 > n3), true_batch_type>::value, "n12 > n3");
+
+        static_assert(std::is_same<decltype(n12 <= n12), true_batch_type>::value, "n12 <= n12");
+        static_assert(std::is_same<decltype(n12 <= n3), false_batch_type>::value, "n12 <= n3");
+
+        static_assert(std::is_same<decltype(n12 >= n12), true_batch_type>::value, "n12 >= n12");
+        static_assert(std::is_same<decltype(n12 >= n3), true_batch_type>::value, "n12 >= n3");
     }
 };
 
 TEST_CASE_TEMPLATE("[constant batch]", B, BATCH_INT_TYPES)
 {
     constant_batch_test<B> Test;
+    SUBCASE("init_from_constant") { Test.test_init_from_constant(); }
+
     SUBCASE("init_from_generator") { Test.test_init_from_generator(); }
 
+    SUBCASE("as_batch") { Test.test_cast(); }
+
     SUBCASE("init_from_generator_arange")
     {
         Test.test_init_from_generator_arange();
     }
 
-    SUBCASE("init_from_constant") { Test.test_init_from_constant(); }
+    SUBCASE("init_from_constant_generator") { Test.test_init_from_constant_generator(); }
 
     SUBCASE("operators")
     {
@@ -152,6 +208,7 @@ struct constant_bool_batch_test
 {
     using batch_type = B;
     using value_type = typename B::value_type;
+    using arch_type = typename B::arch_type;
     static constexpr size_t size = B::size;
     using array_type = std::array<value_type, size>;
     using bool_array_type = std::array<bool, size>;
@@ -165,6 +222,16 @@ struct constant_bool_batch_test
         }
     };
 
+    void test_init_from_constant() const
+    {
+        bool_array_type expected;
+        std::generate(expected.begin(), expected.end(), []()
+                      { return false; });
+        constexpr auto b = xsimd::make_batch_bool_constant<value_type, false, arch_type>();
+        INFO("batch_bool_constant(value_type)");
+        CHECK_BATCH_EQ((batch_bool_type)b, expected);
+    }
+
     void test_init_from_generator() const
     {
         bool_array_type expected;
@@ -172,7 +239,7 @@ struct constant_bool_batch_test
         std::generate(expected.begin(), expected.end(),
                       [&i]()
                       { return generator::get(i++, size); });
-        constexpr auto b = xsimd::make_batch_bool_constant<batch_type, generator>();
+        constexpr auto b = xsimd::make_batch_bool_constant<value_type, generator, arch_type>();
         INFO("batch_bool_constant(value_type)");
         CHECK_BATCH_EQ((batch_bool_type)b, expected);
     }
@@ -192,7 +259,7 @@ struct constant_bool_batch_test
         std::generate(expected.begin(), expected.end(),
                       [&i]()
                       { return split::get(i++, size); });
-        constexpr auto b = xsimd::make_batch_bool_constant<batch_type, split>();
+        constexpr auto b = xsimd::make_batch_bool_constant<value_type, split, arch_type>();
         INFO("batch_bool_constant(value_type)");
         CHECK_BATCH_EQ((batch_bool_type)b, expected);
     }
@@ -214,13 +281,22 @@ struct constant_bool_batch_test
         }
     };
 
+    void test_cast() const
+    {
+        constexpr auto all_true = xsimd::make_batch_bool_constant<value_type, constant<true>, arch_type>();
+        auto b0 = all_true.as_batch_bool();
+        auto b1 = (batch_bool_type)all_true;
+        CHECK_BATCH_EQ(b0, batch_bool_type(true));
+        CHECK_BATCH_EQ(b1, batch_bool_type(true));
+    }
+
     void test_ops() const
     {
-        constexpr auto all_true = xsimd::make_batch_bool_constant<batch_type, constant<true>>();
-        constexpr auto all_false = xsimd::make_batch_bool_constant<batch_type, constant<false>>();
+        constexpr auto all_true = xsimd::make_batch_bool_constant<value_type, constant<true>, arch_type>();
+        constexpr auto all_false = xsimd::make_batch_bool_constant<value_type, constant<false>, arch_type>();
 
-        constexpr auto x = xsimd::make_batch_bool_constant<batch_type, split>();
-        constexpr auto y = xsimd::make_batch_bool_constant<batch_type, inv_split>();
+        constexpr auto x = xsimd::make_batch_bool_constant<value_type, split, arch_type>();
+        constexpr auto y = xsimd::make_batch_bool_constant<value_type, inv_split, arch_type>();
 
         constexpr auto x_or_y = x | y;
         static_assert(std::is_same<decltype(x_or_y), decltype(all_true)>::value, "x | y == true");
@@ -248,8 +324,12 @@ struct constant_bool_batch_test
 TEST_CASE_TEMPLATE("[constant bool batch]", B, BATCH_INT_TYPES)
 {
     constant_bool_batch_test<B> Test;
+    SUBCASE("init_from_constant") { Test.test_init_from_constant(); }
+
     SUBCASE("init_from_generator") { Test.test_init_from_generator(); }
 
+    SUBCASE("as_batch") { Test.test_cast(); }
+
     SUBCASE("init_from_generator_split")
     {
         Test.test_init_from_generator_split();
