@@ -15,9 +15,8 @@
 #include "test_utils.hpp"
 
 template <class B>
-class exponential_test : public testing::Test
+struct exponential_test
 {
-protected:
     using batch_type = B;
     using value_type = typename B::value_type;
     static constexpr size_t size = B::size;
@@ -58,7 +57,8 @@ protected:
                 detail::store_batch(out, res, i);
             }
             size_t diff = detail::get_nb_diff(res, expected);
-            EXPECT_EQ(diff, 0) << print_function_name("exp");
+            INFO("exp");
+            CHECK_EQ(diff, 0);
         }
 
         // exp2
@@ -74,7 +74,8 @@ protected:
                 detail::store_batch(out, res, i);
             }
             size_t diff = detail::get_nb_diff(res, expected);
-            EXPECT_EQ(diff, 0) << print_function_name("exp2");
+            INFO("exp2");
+            CHECK_EQ(diff, 0);
         }
 
         // exp10
@@ -91,7 +92,8 @@ protected:
                 detail::store_batch(out, res, i);
             }
             size_t diff = detail::get_nb_diff(res, expected);
-            EXPECT_EQ(diff, 0) << print_function_name("exp10");
+            INFO("exp10");
+            CHECK_EQ(diff, 0);
         }
 
         // expm1
@@ -107,7 +109,8 @@ protected:
                 detail::store_batch(out, res, i);
             }
             size_t diff = detail::get_nb_diff(res, expected);
-            EXPECT_EQ(diff, 0) << print_function_name("expm1");
+            INFO("expm1");
+            CHECK_EQ(diff, 0);
         }
     }
 
@@ -126,7 +129,8 @@ protected:
                 detail::store_batch(out, res, i);
             }
             size_t diff = detail::get_nb_diff(res, expected);
-            EXPECT_EQ(diff, 0) << print_function_name("log");
+            INFO("log");
+            CHECK_EQ(diff, 0);
         }
 
         // log2
@@ -142,7 +146,8 @@ protected:
                 detail::store_batch(out, res, i);
             }
             size_t diff = detail::get_nb_diff(res, expected);
-            EXPECT_EQ(diff, 0) << print_function_name("log2");
+            INFO("log2");
+            CHECK_EQ(diff, 0);
         }
 
         // log10
@@ -158,7 +163,8 @@ protected:
                 detail::store_batch(out, res, i);
             }
             size_t diff = detail::get_nb_diff(res, expected);
-            EXPECT_EQ(diff, 0) << print_function_name("log10");
+            INFO("log10");
+            CHECK_EQ(diff, 0);
         }
 
         // log1p
@@ -174,20 +180,23 @@ protected:
                 detail::store_batch(out, res, i);
             }
             size_t diff = detail::get_nb_diff(res, expected);
-            EXPECT_EQ(diff, 0) << print_function_name("log1p");
+            CHECK_EQ(diff, 0);
         }
     }
 };
 
-TYPED_TEST_SUITE(exponential_test, batch_float_types, simd_test_names);
-
-TYPED_TEST(exponential_test, exp)
+TEST_CASE_TEMPLATE("[exponential]", B, BATCH_FLOAT_TYPES)
 {
-    this->test_exponential_functions();
-}
+    exponential_test<B> Test;
 
-TYPED_TEST(exponential_test, log)
-{
-    this->test_log_functions();
+    SUBCASE("exp")
+    {
+        Test.test_exponential_functions();
+    }
+
+    SUBCASE("log")
+    {
+        Test.test_log_functions();
+    }
 }
 #endif
