@@ -14,6 +14,13 @@
 
 #include "test_utils.hpp"
 
+#ifdef __linux__
+#include "endian.h"
+#if BYTE_ORDER == BIG_ENDIAN
+#define XSIMD_NO_SLIDE
+#endif
+#endif
+
 #include <numeric>
 
 namespace
@@ -32,8 +39,8 @@ namespace
             /* Generate input data: lhs, rhs */
             for (size_t i = 0; i < N; ++i)
             {
-                lhs_in[i] = 'A' + 2 * i + 1;
-                rhs_in[i] = 'A' + 2 * i;
+                lhs_in[i] = static_cast<T>('A' + 2 * i + 1);
+                rhs_in[i] = static_cast<T>('A' + 2 * i);
             }
             vects.push_back(std::move(lhs_in));
             vects.push_back(std::move(rhs_in));
@@ -166,7 +173,6 @@ namespace
     };
 }
 
-#if !XSIMD_WITH_AVX512F || XSIMD_WITH_AVX512BW
 template <class B>
 struct slide_test : public init_slide_base<typename B::value_type, B::size>
 {
@@ -198,6 +204,7 @@ struct slide_test : public init_slide_base<typename B::value_type, B::size>
         INFO("slide_left full");
         CHECK_BATCH_EQ(b_res_left_full, b_left_full);
 
+#ifndef XSIMD_NO_SLIDE
         B b_res_left_half = xsimd::slide_left<half_slide>(b_in);
         INFO("slide_left half_slide");
         CHECK_BATCH_EQ(b_res_left_half, b_left_half);
@@ -216,6 +223,7 @@ struct slide_test : public init_slide_base<typename B::value_type, B::size>
             INFO("slide_left below_half_slide");
             CHECK_BATCH_EQ(b_res_left_below_half, b_left_below_half);
         }
+#endif
     }
 
     void slide_right()
@@ -236,6 +244,7 @@ struct slide_test : public init_slide_base<typename B::value_type, B::size>
         INFO("slide_right full");
         CHECK_BATCH_EQ(b_res_right_full, b_right_full);
 
+#ifndef XSIMD_NO_SLIDE
         B b_res_right_half = xsimd::slide_right<half_slide>(b_in);
         INFO("slide_right half_slide");
         CHECK_BATCH_EQ(b_res_right_half, b_right_half);
@@ -254,6 +263,7 @@ struct slide_test : public init_slide_base<typename B::value_type, B::size>
             INFO("slide_right below_half_slide");
             CHECK_BATCH_EQ(b_res_right_below_half, b_right_below_half);
         }
+#endif
     }
 };
 
@@ -270,8 +280,6 @@ TEST_CASE_TEMPLATE("[slide]", B, BATCH_INT_TYPES)
     }
 }
 
-#endif
-
 template <class B>
 struct compress_test
 {
@@ -288,7 +296,7 @@ struct compress_test
     {
         for (size_t i = 0; i < size; ++i)
         {
-            input[i] = i;
+            input[i] = static_cast<value_type>(i);
         }
     }
 
@@ -332,7 +340,7 @@ struct compress_test
         CHECK_BATCH_EQ(b, expected);
     }
 
-    void generic()
+    void common()
     {
         for (size_t i = 0; i < size; ++i)
             mask[i] = i % 3 == 0;
@@ -347,7 +355,9 @@ struct compress_test
     }
 };
 
-TEST_CASE_TEMPLATE("[compress]", B, BATCH_FLOAT_TYPES, xsimd::batch<uint32_t>, xsimd::batch<int32_t>, xsimd::batch<uint64_t>, xsimd::batch<int64_t>)
+#define XSIMD_COMPRESS_TYPES BATCH_FLOAT_TYPES, xsimd::batch<uint8_t>, xsimd::batch<int8_t>, xsimd::batch<uint16_t>, xsimd::batch<int16_t>, xsimd::batch<uint32_t>, xsimd::batch<int32_t>, xsimd::batch<uint64_t>, xsimd::batch<int64_t>
+
+TEST_CASE_TEMPLATE("[compress]", B, XSIMD_COMPRESS_TYPES)
 {
     compress_test<B> Test;
     SUBCASE("empty")
@@ -360,11 +370,11 @@ TEST_CASE_TEMPLATE("[compress]", B, BATCH_FLOAT_TYPES, xsimd::batch<uint32_t>, x
     }
     // SUBCASE("interleave")
     //{
-    //     Test.interleave();
+    //      Test.interleave();
     // }
-    // SUBCASE("generic")
+    // SUBCASE("common")
     //{
-    //     Test.generic();
+    //      Test.common();
     // }
 }
 
@@ -384,7 +394,7 @@ struct expand_test
     {
         for (size_t i = 0; i < size; ++i)
         {
-            input[i] = i;
+            input[i] = static_cast<value_type>(i);
         }
     }
 
@@ -428,7 +438,7 @@ struct expand_test
         CHECK_BATCH_EQ(b, expected);
     }
 
-    void generic()
+    void common()
     {
         for (size_t i = 0; i < size; ++i)
             mask[i] = i % 3 == 0;
@@ -443,7 +453,9 @@ struct expand_test
     }
 };
 
-TEST_CASE_TEMPLATE("[expand]", B, BATCH_FLOAT_TYPES, xsimd::batch<uint32_t>, xsimd::batch<int32_t>, xsimd::batch<uint64_t>, xsimd::batch<int64_t>)
+#define XSIMD_EXPAND_TYPES XSIMD_COMPRESS_TYPES
+
+TEST_CASE_TEMPLATE("[expand]", B, XSIMD_EXPAND_TYPES)
 {
     expand_test<B> Test;
     SUBCASE("empty")
@@ -458,9 +470,9 @@ TEST_CASE_TEMPLATE("[expand]", B, BATCH_FLOAT_TYPES, xsimd::batch<uint32_t>, xsi
     {
         Test.interleave();
     }
-    SUBCASE("generic")
+    SUBCASE("common")
     {
-        Test.generic();
+        Test.common();
     }
 }
 
@@ -479,8 +491,8 @@ struct shuffle_test
     {
         for (size_t i = 0; i < size; ++i)
         {
-            lhs[i] = i;
-            rhs[i] = i + size;
+            lhs[i] = static_cast<value_type>(i);
+            rhs[i] = static_cast<value_type>(i + size);
         }
     }
 
@@ -498,7 +510,7 @@ struct shuffle_test
         };
 
         INFO("no op lhs");
-        B b_res_lhs = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, arch_type, no_op_lhs_generator>());
+        B b_res_lhs = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, no_op_lhs_generator, arch_type>());
         CHECK_BATCH_EQ(b_res_lhs, b_lhs);
 
         struct no_op_rhs_generator
@@ -510,16 +522,16 @@ struct shuffle_test
         };
 
         INFO("no op rhs");
-        B b_res_rhs = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, arch_type, no_op_rhs_generator>());
+        B b_res_rhs = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, no_op_rhs_generator, arch_type>());
         CHECK_BATCH_EQ(b_res_rhs, b_rhs);
     }
 
-    void generic()
+    void common()
     {
         B b_lhs = B::load_unaligned(lhs.data());
         B b_rhs = B::load_unaligned(rhs.data());
 
-        struct generic_generator
+        struct common_generator
         {
             static constexpr size_t get(size_t index, size_t size)
             {
@@ -535,7 +547,7 @@ struct shuffle_test
         }
         B b_ref = B::load_unaligned(ref.data());
 
-        B b_res = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, arch_type, generic_generator>());
+        B b_res = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, common_generator, arch_type>());
         CHECK_BATCH_EQ(b_res, b_ref);
     }
 
@@ -557,17 +569,17 @@ struct shuffle_test
             ref[i] = (i > 2) ? lhs[0] : rhs[0];
         B b_ref = B::load_unaligned(ref.data());
 
-        B b_res = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, arch_type, pick_generator>());
+        B b_res = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, pick_generator, arch_type>());
         CHECK_BATCH_EQ(b_res, b_ref);
     }
 
-    void swizzle()
+    void shuffle()
     {
         B b_lhs = B::load_unaligned(lhs.data());
         B b_rhs = B::load_unaligned(rhs.data());
 
         {
-            struct swizzle_lo_generator
+            struct shuffle_lo_generator
             {
                 static constexpr size_t get(size_t index, size_t size)
                 {
@@ -580,13 +592,13 @@ struct shuffle_test
                 ref[i] = lhs[size - i - 1];
             B b_ref = B::load_unaligned(ref.data());
 
-            INFO("swizzle first batch");
-            B b_res = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, arch_type, swizzle_lo_generator>());
+            INFO("shuffle first batch");
+            B b_res = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, shuffle_lo_generator, arch_type>());
             CHECK_BATCH_EQ(b_res, b_ref);
         }
 
         {
-            struct swizzle_hi_generator
+            struct shuffle_hi_generator
             {
                 static constexpr size_t get(size_t index, size_t size)
                 {
@@ -599,8 +611,8 @@ struct shuffle_test
                 ref[i] = rhs[size - i - 1];
             B b_ref = B::load_unaligned(ref.data());
 
-            INFO("swizzle second batch");
-            B b_res = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, arch_type, swizzle_hi_generator>());
+            INFO("shuffle second batch");
+            B b_res = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, shuffle_hi_generator, arch_type>());
             CHECK_BATCH_EQ(b_res, b_ref);
         }
     }
@@ -643,7 +655,7 @@ struct shuffle_test
         B b_ref = B::load_unaligned(ref.data());
 
         INFO("select");
-        B b_res = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, arch_type, select_generator>());
+        B b_res = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, select_generator, arch_type>());
         CHECK_BATCH_EQ(b_res, b_ref);
     }
 
@@ -666,7 +678,7 @@ struct shuffle_test
         B b_ref_lo = B::load_unaligned(ref_lo.data());
 
         INFO("zip_lo");
-        B b_res_lo = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, arch_type, zip_lo_generator>());
+        B b_res_lo = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, zip_lo_generator, arch_type>());
         CHECK_BATCH_EQ(b_res_lo, b_ref_lo);
 
         struct zip_hi_generator
@@ -685,21 +697,23 @@ struct shuffle_test
         B b_ref_hi = B::load_unaligned(ref_hi.data());
 
         INFO("zip_hi");
-        B b_res_hi = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, arch_type, zip_hi_generator>());
+        B b_res_hi = xsimd::shuffle(b_lhs, b_rhs, xsimd::make_batch_constant<mask_type, zip_hi_generator, arch_type>());
         CHECK_BATCH_EQ(b_res_hi, b_ref_hi);
     }
 };
 
-TEST_CASE_TEMPLATE("[shuffle]", B, BATCH_FLOAT_TYPES, xsimd::batch<uint32_t>, xsimd::batch<int32_t>, xsimd::batch<uint64_t>, xsimd::batch<int64_t>)
+#define XSIMD_SHUFFLE_TYPES BATCH_FLOAT_TYPES, xsimd::batch<uint32_t>, xsimd::batch<int32_t>, xsimd::batch<uint64_t>, xsimd::batch<int64_t>
+
+TEST_CASE_TEMPLATE("[shuffle]", B, XSIMD_SHUFFLE_TYPES)
 {
     shuffle_test<B> Test;
     SUBCASE("no-op")
     {
         Test.no_op();
     }
-    SUBCASE("generic")
+    SUBCASE("common")
     {
-        Test.generic();
+        Test.common();
     }
     SUBCASE("pick")
     {
@@ -709,9 +723,9 @@ TEST_CASE_TEMPLATE("[shuffle]", B, BATCH_FLOAT_TYPES, xsimd::batch<uint32_t>, xs
     {
         Test.select();
     }
-    SUBCASE("swizzle")
+    SUBCASE("shuffle")
     {
-        Test.swizzle();
+        Test.shuffle();
     }
     SUBCASE("transpose")
     {
@@ -723,4 +737,24 @@ TEST_CASE_TEMPLATE("[shuffle]", B, BATCH_FLOAT_TYPES, xsimd::batch<uint32_t>, xs
     }
 }
 
+TEST_CASE_TEMPLATE("[small integer transpose]", B, xsimd::batch<uint16_t>, xsimd::batch<int16_t>, xsimd::batch<uint8_t>, xsimd::batch<int8_t>)
+{
+    shuffle_test<B> Test;
+    SUBCASE("transpose")
+    {
+        Test.transpose();
+    }
+}
+
+#if (XSIMD_WITH_SSE2 && !XSIMD_WITH_AVX)
+TEST_CASE_TEMPLATE("[small integer shuffle]", B, xsimd::batch<uint16_t>, xsimd::batch<int16_t>)
+{
+    shuffle_test<B> Test;
+    SUBCASE("shuffle")
+    {
+        Test.shuffle();
+    }
+}
+#endif
+
 #endif
