@@ -353,18 +353,28 @@ struct xsimd_api_integral_types_functions
 
     void test_bitwise_lshift()
     {
+        constexpr int shift = 3;
         value_type val0(12);
-        value_type val1(3);
+        value_type val1(shift);
         value_type r = val0 << val1;
+        value_type ir = val0 << shift;
+        T cr = xsimd::bitwise_lshift<shift>(T(val0));
         CHECK_EQ(extract(xsimd::bitwise_lshift(T(val0), T(val1))), r);
+        CHECK_EQ(extract(ir), r);
+        CHECK_EQ(extract(cr), r);
     }
 
     void test_bitwise_rshift()
     {
+        constexpr int shift = 3;
         value_type val0(12);
-        value_type val1(3);
+        value_type val1(shift);
         value_type r = val0 >> val1;
+        value_type ir = val0 >> shift;
+        T cr = xsimd::bitwise_rshift<shift>(T(val0));
         CHECK_EQ(extract(xsimd::bitwise_rshift(T(val0), T(val1))), r);
+        CHECK_EQ(extract(ir), r);
+        CHECK_EQ(extract(cr), r);
     }
 
     void test_mod()
@@ -376,20 +386,26 @@ struct xsimd_api_integral_types_functions
 
     void test_rotl()
     {
-        constexpr auto N = std::numeric_limits<value_type>::digits;
+        constexpr auto N = std::numeric_limits<value_type>::digits + std::numeric_limits<value_type>::is_signed;
+        constexpr int count = 3;
         value_type val0(12);
-        value_type val1(3);
+        value_type val1(count);
         value_type r = (val0 << val1) | (val0 >> (N - val1));
+        T cr = xsimd::rotl<count>(T(val0));
         CHECK_EQ(extract(xsimd::rotl(T(val0), T(val1))), r);
+        CHECK_EQ(extract(cr), r);
     }
 
     void test_rotr()
     {
-        constexpr auto N = std::numeric_limits<value_type>::digits;
+        constexpr auto N = std::numeric_limits<value_type>::digits + std::numeric_limits<value_type>::is_signed;
+        constexpr int count = 3;
         value_type val0(12);
-        value_type val1(3);
+        value_type val1(count);
         value_type r = (val0 >> val1) | (val0 << (N - val1));
+        T cr = xsimd::rotr<3>(T(val0));
         CHECK_EQ(extract(xsimd::rotr(T(val0), T(val1))), r);
+        CHECK_EQ(extract(cr), r);
     }
 
     void test_sadd()
@@ -468,7 +484,11 @@ struct xsimd_api_float_types_functions
     void test_asin()
     {
         value_type val(1);
+#if defined(__APPLE__)
+        CHECK_EQ(extract(xsimd::asin(T(val))), doctest::Approx(std::asin(val)));
+#else
         CHECK_EQ(extract(xsimd::asin(T(val))), std::asin(val));
+#endif
     }
     void test_asinh()
     {
@@ -488,8 +508,12 @@ struct xsimd_api_float_types_functions
     }
     void test_atanh()
     {
-        value_type val(1);
-        CHECK_EQ(extract(xsimd::atanh(T(val))), std::atanh(val));
+        value_type val0(0);
+        CHECK_EQ(extract(xsimd::atanh(T(val0))), std::atanh(val0));
+#ifndef __FAST_MATH__
+        value_type val1(1);
+        CHECK_EQ(extract(xsimd::atanh(T(val1))), std::atanh(val1));
+#endif
     }
     void test_cbrt()
     {
@@ -623,7 +647,8 @@ struct xsimd_api_float_types_functions
     {
         value_type val0(4);
         xsimd::as_integer_t<value_type> val1(2);
-        CHECK_EQ(extract(xsimd::ldexp(T(val0), xsimd::as_integer_t<T>(val1))), std::ldexp(val0, val1));
+        using exponent_type = typename std::conditional<std::is_scalar<T>::value, int, xsimd::as_integer_t<T>>::type;
+        CHECK_EQ(extract(xsimd::ldexp(T(val0), exponent_type(val1))), std::ldexp(val0, static_cast<int>(val1)));
     }
     void test_lgamma()
     {
@@ -653,6 +678,29 @@ struct xsimd_api_float_types_functions
         value_type val(0);
         CHECK_EQ(extract(xsimd::log1p(T(val))), std::log1p(val));
     }
+
+    void test_max_nan()
+    {
+        value_type val0(2.7818);
+        value_type valN(NAN);
+#if XSIMD_WITH_AVX || XSIMD_WITH_SSE2
+        using isnan = doctest::IsNaN<value_type>;
+        CHECK_EQ(isnan(extract(xsimd::max(T(val0), T(valN)))), isnan(std::max(val0, valN)));
+        CHECK_EQ(isnan(extract(xsimd::max(T(valN), T(val0)))), isnan(std::max(valN, val0)));
+#endif
+    }
+
+    void test_min_nan()
+    {
+        value_type val0(2.7818);
+        value_type valN(NAN);
+#if XSIMD_WITH_AVX || XSIMD_WITH_SSE2
+        using isnan = doctest::IsNaN<value_type>;
+        CHECK_EQ(isnan(extract(xsimd::min(T(val0), T(valN)))), isnan(std::min(val0, valN)));
+        CHECK_EQ(isnan(extract(xsimd::min(T(valN), T(val0)))), isnan(std::min(valN, val0)));
+#endif
+    }
+
     void test_nearbyint()
     {
         value_type val(3.1);
@@ -924,6 +972,16 @@ TEST_CASE_TEMPLATE("[xsimd api | float types functions]", B, FLOAT_TYPES)
         Test.test_log1p();
     }
 
+    SUBCASE("max_nan")
+    {
+        Test.test_max_nan();
+    }
+
+    SUBCASE("min_nan")
+    {
+        Test.test_min_nan();
+    }
+
     SUBCASE("nearbyint")
     {
         Test.test_nearbyint();
@@ -1043,6 +1101,7 @@ struct xsimd_api_complex_types_functions
         CHECK_EQ(extract(xsimd::proj(T(val))), std::proj(val));
     }
 
+#ifndef __FAST_MATH__
     void test_isinf()
     {
         value_type val(4);
@@ -1060,6 +1119,7 @@ struct xsimd_api_complex_types_functions
         value_type val(4);
         CHECK_EQ(extract(xsimd::isnan(T(val))), std::isnan(std::real(val)));
     }
+#endif
 };
 
 TEST_CASE_TEMPLATE("[xsimd api | complex types functions]", B, COMPLEX_TYPES)
@@ -1085,6 +1145,8 @@ TEST_CASE_TEMPLATE("[xsimd api | complex types functions]", B, COMPLEX_TYPES)
         Test.test_proj();
     }
 
+#ifndef __FAST_MATH__
+
     SUBCASE("isinf")
     {
         Test.test_isinf();
@@ -1099,6 +1161,7 @@ TEST_CASE_TEMPLATE("[xsimd api | complex types functions]", B, COMPLEX_TYPES)
     {
         Test.test_isnan();
     }
+#endif
 }
 
 /*
@@ -1193,7 +1256,7 @@ struct xsimd_api_all_types_functions
     void test_div()
     {
         value_type val0(1);
-        value_type val1(3);
+        value_type val1(2);
         CHECK_EQ(extract(xsimd::div(T(val0), T(val1))), val0 / val1);
     }
 
@@ -1344,6 +1407,7 @@ TEST_CASE_TEMPLATE("[xsimd api | all types functions]", B, ALL_TYPES)
 /*
  * Functions that apply only to floating point types
  */
+#ifndef __FAST_MATH__
 template <typename T>
 struct xsimd_api_all_floating_point_types_functions
 {
@@ -1362,6 +1426,7 @@ TEST_CASE_TEMPLATE("[xsimd api | all floating point types functions]", B, ALL_FL
     xsimd_api_all_floating_point_types_functions<B> Test;
     Test.test_neq_nan();
 }
+#endif
 
 /*
  * Functions that apply only to mask type
